@@ -1,5 +1,11 @@
 package C::DynaLib::Struct;
 require 5.002;
+use C::DynaLib;
+BEGIN {
+  if ($] > 5.008) {
+    require C::DynaLib::Parse;
+  }
+}
 
 =head1 NAME
 
@@ -15,7 +21,7 @@ C::DynaLib::Struct - Tool for handling the C `struct' data type
 	[$template1, \@field_names1,]
 	... );
 
-  Parse C::DynaLib::Struct <<ENDC;
+  C::DynaLib::Struct::Parse <<ENDC;
   struct packet {
     unsigned short header;
     unsigned short flags;
@@ -103,9 +109,14 @@ L<Convert::Binary::C>, L<perlfunc(1)> (for C<pack>), L<perlref(1)>, L<perltie(1)
 =cut
 
 use strict qw (vars subs);
-use vars qw($VERSION);
-$VERSION = '0.59';
 use subs qw(Define Parse);
+if ($] > 5.008) {
+  *C::DynaLib::Struct::pack_types     = *C::DynaLib::Parse::pack_types;
+  *C::DynaLib::Struct::declare_func   = *C::DynaLib::Parse::declare_func;
+  *C::DynaLib::Struct::declare_struct = *C::DynaLib::Parse::declare_struct;
+  *C::DynaLib::Struct::process_func   = *C::DynaLib::Parse::process_func;
+  *C::DynaLib::Struct::process_struct = *C::DynaLib::Parse::process_struct;
+}
 
 package C::DynaLib::Struct::Imp;
 
@@ -211,28 +222,69 @@ sub Define {
 
 sub Parse {
   my $definition = shift;
+  $definition = shift if $definition eq 'C::DynaLib::Struct';
   my $c;
-  require Convert::Binary::C;
-  Convert::Binary::C->import;
-  if (ref $definition eq 'Convert::Binary::C') {
-    $c = $definition;
-    $c->parse(@_);
+  if (eval "require Convert::Binary::C;") {
+    Convert::Binary::C->import;
+    if (ref $definition eq 'Convert::Binary::C') {
+      $c = $definition;
+      $c->parse(@_);
+    } else {
+      require C::DynaLib::PerlTypes;
+      $c = Convert::Binary::C->new(%$C::DynaLib::PerlTypes::PerlTypes);
+      $c->parse($definition, @_);
+    }
+    # all structs and unions
+    for my $s ($c->compound) {
+      my $record = $s->{identifier};
+      if (defined (${"${record}::template"})) { # already parsed
+	carp "Redefinition of ".$s->{type}." $record\n";
+      }
+      # Convert::Binary::C bug in 0.74
+      # fixup wrong declarations: 'char'(type) '*baz'(declarator) => 'char*' 'baz'
+      for (0..@{$s->{declarations}}) {
+	  my $d = $s->{declarations}->[$_];
+	  if ($d and $d->{declarators}[0]->{declarator} 
+	      and substr($d->{declarators}[0]->{declarator},0,1) eq '*')
+	  {
+	      $s->{declarations}->[$_]->{declarators}[0]->{declarator} =
+		  substr($d->{declarators}[0]->{declarator},1);
+	      $s->{declarations}->[$_]->{type} .= "*";
+	  }
+      }
+      my @members = _members(@{$s->{declarations}});
+      Define C::DynaLib::Struct($record,
+				_pack_names(@{$s->{declarations}}),
+				\@members);
+    }
   } else {
-    require C::DynaLib::PerlTypes;
-    my $c = new Convert::Binary::C $C::DynaLib::PerlTypes::PerlTypes;
-    $c->parse($definition, @_);
-  }
-  # all structs and unions
-  for my $s ($c->compound) {
-    my $class = $s->identifier;
-    if (defined (${"${class}::template"})) {
-      carp "Redefinition of ".$s->type." $class";
+    # XXX use GCC::TranslationUnit (does not work yet)
+    my $node = C::DynaLib::Parse::GCC_prepare($definition);
+    while ($node) {
+      if ($node->isa('GCC::Node::function_decl')) {
+	declare_func(process_func($node));
+      }
+      if ($node->isa('GCC::Node::record_type')) {
+	declare_struct(process_struct($node));
+      }
+    } continue {
+      $node = $node->chain;
+    }
+  POST:
+    while ($node = shift @C::DynaLib::Parse::post) {
+      if ($node->isa('GCC::Node::record_type')) {
+	declare_struct(process_struct($node));
+      }
     }
-    my @members = $s->members;
-    Define C::DynaLib::Struct($class, $s->packnames, \@members);
   }
-  use Data::Dumper;
-  print Dumper($c);
+}
+
+sub _members {
+  map {my $decl=$_->{declarators}[0]->{declarator}; $decl=~s/^\*//; $decl} @_;
+}
+
+sub _pack_names {
+  C::DynaLib::Parse::pack_types (map {$_->{type}} @_);
 }
 
 1;
